package dev.peerat.parser.java;

import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Function;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class Annotation extends JavaElement{
	
	public static interface AnnotableBuffer{
		
		void addAnnotationBuffer(Annotation annotation);
		
		List<Annotation> getAnnotationBuffer();
		
	}
	
	public static abstract class Annotable extends JavaElement{
		
		private List<Annotation> annotations;
		
		public Annotable(List<Annotation> annotations){
			this.annotations = annotations;
		}
		
		public List<Annotation> getAnnotations(){
			return this.annotations;
		}
		
		public Annotation findAnnotation(Function<Annotation, Boolean> finder){
			if(annotations != null){
				for(Annotation annotation : this.annotations){
					if(finder.apply(annotation)) return annotation;
				}
			}
			return null;
		}
		
		public boolean hasAnnotation(Function<Annotation, Boolean> finder){
			if(annotations != null){
				for(Annotation annotation : this.annotations){
					if(finder.apply(annotation)) return true;
				}
			}
			return false;
		}
		
		@Override
		public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
			if(annotations != null){
				for(Annotation annotation : this.annotations){
					if(finder.apply(annotation)) return (E) annotation;
				}
			}
			return null;
		}
		
		@Override
		public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
			if(annotations != null){
				for(Annotation annotation : this.annotations){
					if(finder.apply(annotation)) list.add((E) annotation);
					annotation.findAll(finder, list);
				}
			}
		}
	}
	
	private Token name;
	private Map<Token, Value> values; //TODO works with key null ?
	
	public Annotation(Token name){
		this.name = name;
	}
	
	public Annotation(Token name, Map<Token, Value> values){
		this(name);
		this.values =  values;
	}
	
	public Token getName(){
		return name;
	}
	
	public Map<Token, Value> getParameters(){
		return this.values;
	}
	
	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
		if(this.values != null){
			for(Value value : this.values.values()){
				if(finder.apply(value)) return (E)value;
			}
		}
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
		if(this.values != null){
			for(Value value : this.values.values()){
				if(finder.apply(value)) list.add((E)value);
				value.findAll(finder, list);
			}
		}
	}

	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		
		if(this.values != null)
			for(Value value : this.values.values()){
				bag.merge(value.visit(visitor));
			}
		
		return bag;
	}

}
