package dev.peerat.parser.java.builder;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.operation.Operation;
import dev.peerat.parser.java.value.Value;

public abstract class JavaBuilder<T extends JavaElement>{
	
	//TODO WHEN TAKE BUILDER, TAKE ALSO BUILDED TYPE
	
	public static JavaFileBuilder ofFile(){
		return new JavaFileBuilder();
	}
	
	public static JavaFileBuilder ofFile(String pack){
		return ofFile().setPackage(pack);
	}
	
	public static JavaImportBuilder ofImport(){
		return new JavaImportBuilder();
	}
	
	public static JavaImportBuilder ofImport(boolean isStatic, String value){
		JavaImportBuilder builder = ofImport().setImport(value);
		return isStatic ? builder.setStatic() : builder;
	}
	
	public static JavaClassBuilder ofClass(){
		return new JavaClassBuilder();
	}
	
	public static JavaClassBuilder ofClass(String name){
		return ofClass().setName(name);
	}
	
	public static JavaAnnotationBuilder ofAnnotation(){
		return new JavaAnnotationBuilder();
	}
	
	public static JavaAnnotationBuilder ofAnnotation(String name){
		return ofAnnotation().setName(name);
	}
	
	public static JavaAnnotationBuilder ofAnnotation(String name, Value paramter){
		return ofAnnotation(name).uniqueParameter(paramter);
	}
	
	public static JavaVariableBuilder ofVariable(){
		return new JavaVariableBuilder();
	}
	
	public static JavaVariableBuilder ofVariable(String type, String name){
		return ofVariable().setType(type).setName(name);
	}
	
	public static JavaVariableBuilder ofVariable(String type, String name, Value value){
		return ofVariable(type, name).setValue(value);
	}
	
	public static JavaFunctionBuilder ofFunction(){
		return new JavaFunctionBuilder();
	}
	
	public static JavaFunctionBuilder ofFunction(String type, String name){
		return ofFunction().setType(type).setName(name);
	}
	
	public static JavaFunctionBuilder ofFunction(String generic, String type, String name){
		return ofFunction(type, name).setGeneric(generic);
	}
	
	public static Operation ofOperation(String operation){
		try{
			return new JavaOperationBuilder(operation).build();
		}catch(Exception e){
			throw new IllegalArgumentException("Failed to parse Operation "+operation, e);
		}
	}
	
	public static Value ofValue(String value){
		try{
			return new JavaValueBuilder(value).build();
		}catch(Exception e){
			throw new IllegalArgumentException("Failed to parse Value "+value, e);
		}
	}
	
	public abstract T build();
	
}
