package dev.peerat.parser.java.builder;

import java.util.ArrayList;
import java.util.List;
import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.value.Value;

public class JavaVariableBuilder extends JavaModifiableBuilder<JavaVariableBuilder, Variable>{

	private List<Annotation> annotations;
	private String type;
	private String name;
	private boolean elipse;
	private Value value;
	
	public JavaVariableBuilder annotate(JavaAnnotationBuilder annotation){
		if(this.annotations == null) this.annotations = new ArrayList<>();
		this.annotations.add(annotation.build());
		return this;
	}
	
	public JavaVariableBuilder setType(String type){
		this.type = type;
		this.elipse = type.endsWith("...");
		return this;
	}
	
	public JavaVariableBuilder setName(String name){
		this.name = name;
		return this;
	}
	
	public JavaVariableBuilder setValue(Value value){
		this.value = value;
		return this;
	}
	
	public JavaVariableBuilder normalize(Variable variable){
		normalizeAnnotations(variable);
		setType(variable.getType().getValue());
		setName(variable.getName().getValue());
		setValue(JavaValueBuilder.normalize(variable.getValue()));
		return this;
	}
	
	public JavaVariableBuilder normalizeAnnotations(Variable variable){
		List<Annotation> annotations = variable.getAnnotations();
		if(annotations != null && annotations.size() > 0){
			if(this.annotations == null) this.annotations = new ArrayList<>();
			for(Annotation annotation : annotations){
				this.annotations.add(JavaBuilder.ofAnnotation().normalize(annotation).build());
			}
		}
		return this;
	}
	
	public JavaVariableBuilder clearAnnotations(){
		this.annotations = null;
		return this;
	}
	
	@Override
	public Variable build(){
		return new Variable(annotations, mod, new Token(0,0, type, null), new Token(0,0, name, null), elipse, value);
	}

}
