package dev.peerat.parser.java.printer;

import java.lang.reflect.Modifier;
import java.util.Iterator;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.value.Value;

public class JavaFunctionPrinter extends JavaPrinter<Function>{
	
	@Override
	public void print(Function element, Writer writer, String buffer, JavaPrintProvider provider) throws Exception {
		writer.write(buffer+Modifier.toString(element.getModifier())+" ");
		if(element.getGeneric() != null){
			writer.write(element.getGeneric());
			writer.write(" ");
		}
		if(element.getReturnType() != null){
			writer.write(element.getReturnType());
			writer.write(" ");
		}
		writer.write(element.getName());
		writer.write("(");
		if(element.getParameters() != null){
			Iterator<Variable> parameters = element.getParameters().iterator();
			while(parameters.hasNext()){
				Variable parameter = parameters.next();
				provider.getPrinter(parameter).print(parameter, writer, "", provider);
				if(parameters.hasNext()) writer.write(", ");
			}
		}
		writer.write(")");
		if(element.getThrowables() != null){
			Iterator<Token> throwables = element.getThrowables().iterator();
			if(throwables.hasNext()){
				writer.write(" throws ");
				while(throwables.hasNext()){
					Token token = throwables.next();
					writer.write(token);
					if(throwables.hasNext()) writer.write(",");
				}
			}
		}
		writer.write("{\n");
		for(JavaElement content : element.getElements()){
			provider.getPrinter(content).print(content, writer, buffer+"\t", provider);
			if(content instanceof Value) writer.write(";");
			writer.write("\n");
		}
		writer.write(buffer+"}");
	}

}
