package dev.peerat.parser.java.tree;

import java.util.function.BiConsumer;

import dev.peerat.parser.Bag;
import dev.peerat.parser.Token;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class GenericTree extends SyntaxTree<JavaElement> {

	public GenericTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init() {

		StateTree<JavaElement> type = registery.getNode(TypeTree.class);

		BiConsumer<Bag, Token> concat = (bag, token) -> {
			Token value = bag.get();
			if (value == null)
				bag.set(token);
			else
				bag.set(value.concat(token));
		};

		StateTree<JavaElement> gen = this.current;
		StateTree<JavaElement> type_begin = gen
				.then((validator) -> validator.validate((token) -> token.getValue().equals("<"), concat));
		StateTree<JavaElement> type_generic = type_begin.then(
				new RedirectStateTree<>(type, (global, local) -> global.set(global.<Token>get().concat(local.get()))));
		StateTree<JavaElement> type_generic_end = type_generic
				.then((validator) -> validator.validate((token) -> token.getValue().equals(">"), concat));
		StateTree<JavaElement> type_generic_split = type_generic
				.then((validator) -> validator.validate((token) -> token.getValue().equals(","), concat));
		type_generic_split.then(type_generic);

		type_begin.then(type_generic_end); // remove ?

		StateTree<JavaElement> type_generic_unknow = type_begin
				.then((validator) -> validator.validate((token) -> token.getValue().equals("?"), concat));
		type_generic_unknow.then(type_generic_end);
		type_generic_split.then(type_generic_unknow);
		type_generic_unknow.then(type_generic_split);
		StateTree<JavaElement> type_generic_unknow_extends = type_generic_unknow
				.then((validator) -> validator.validate((token) -> token.getValue().equals("extends"), concat))
				.then(new RedirectStateTree<>(type,
						(global, local) -> global.set(global.<Token>get().concat(local.get()))));
		type_generic_unknow_extends.then(type_generic_end);
		type_generic_unknow_extends.then(type_generic_split);
		StateTree<JavaElement> type_generic_unknow_super = type_generic_unknow
				.then((validator) -> validator.validate((token) -> token.getValue().equals("super"), concat))
				.then(new RedirectStateTree<>(type,
						(global, local) -> global.set(global.<Token>get().concat(local.get()))));
		type_generic_unknow_super.then(type_generic_end);
		type_generic_unknow_super.then(type_generic_split);

		StateTree<JavaElement> type_generic_named_extends = type_generic
				.then((validator) -> validator.validate((token) -> token.getValue().equals("extends"), concat))
				.then(new RedirectStateTree<>(type,
						(global, local) -> global.set(global.<Token>get().concat(local.get()))));
		type_generic_named_extends.then(type_generic_end);
		type_generic_named_extends.then(type_generic_split);
		StateTree<JavaElement> type_generic_named_super = type_generic
				.then((validator) -> validator.validate((token) -> token.getValue().equals("super"), concat))
				.then(new RedirectStateTree<>(type,
						(global, local) -> global.set(global.<Token>get().concat(local.get()))));
		type_generic_named_super.then(type_generic_end);
		type_generic_named_super.then(type_generic_split);

		type_generic_end.end();
	}

}
