package dev.peerat.parser.java.tree;

import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.operation.AssignOperation;
import dev.peerat.parser.java.operation.BreakOperation;
import dev.peerat.parser.java.operation.ContinueOperation;
import dev.peerat.parser.java.operation.ForOperation;
import dev.peerat.parser.java.operation.IfOperation;
import dev.peerat.parser.java.operation.ReturnOperation;
import dev.peerat.parser.java.operation.SynchronizedOperation;
import dev.peerat.parser.java.operation.ThrowOperation;
import dev.peerat.parser.java.operation.WhileOperation;
import dev.peerat.parser.java.operation.Operation.OperationContainer;
import dev.peerat.parser.java.value.BiValue;
import dev.peerat.parser.java.value.MethodCallValue;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class OperationTree extends SyntaxTree<JavaElement> {

	public OperationTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init() {
		StateTree<JavaElement> modifier = registery.getNode(ModifierTree.class);
		StateTree<JavaElement> type = registery.getNode(TypeTree.class);
		StateTree<JavaElement> value = registery.getNode(ValueTree.class);
		StateTree<JavaElement> variable = registery.getNode(VariableTree.class);
		StateTree<JavaElement> function_container = registery.getNode(FunctionContainerTree.class);

		// OPERATION
		StateTree<JavaElement> operation = this.current;

		StateTree<JavaElement> operation_return = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("return")));
		operation_return.then(new RedirectStateTree<>(value, (global, local) -> global.set(local.get())))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end((parent, bag) -> {
					ReturnOperation op = new ReturnOperation(bag.get());
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return null;
				});

		operation_return.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
				.end((parent, bag) -> {
					ReturnOperation op = new ReturnOperation(bag.get());
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return null;
				});

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("throw")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(local.get())))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end((parent, bag) -> {
					ThrowOperation op = new ThrowOperation(bag.get());
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return null;
				});

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("do")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{"))).end((a, b) -> a)
				.multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_else = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("else"))).end((a, b) -> a);
		operation_else.then(operation);
		operation_else.then((validator) -> validator.validate((token) -> token.getValue().equals("{"))).end((a, b) -> a)
				.multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_try = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("try")));
		StateTree<JavaElement> operation_try_base = operation_try
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{")));
		operation_try.then((validator) -> validator.validate((token) -> token.getValue().equals("(")))
				.then(new RedirectStateTree<>(variable, (global, local) -> global.set(null)))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")))
				.then(operation_try_base);
		StateTree<JavaElement> operation_try_end = operation_try_base.end((a, b) -> a).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_finally = operation_try_end
				.then((validator) -> validator.validate((token) -> token.getValue().equals("finally")));
		operation_finally.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
				.end((a, b) -> a).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_catch_named = operation_try_end
				.then((validator) -> validator.validate((token) -> token.getValue().equals("catch")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")))
				.then(new RedirectStateTree<>(type, (global, local) -> global.set(null)));
		operation_catch_named.then((validator) -> validator.validate((token) -> token.getValue().equals("|")))
				.then(operation_catch_named);
		StateTree<JavaElement> operation_catch = operation_catch_named
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME)))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{"))).end((a, b) -> a)
				.multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);
		operation_catch.then(operation_finally);
		operation_catch.then(operation_catch_named);

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("continue")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end((parent, bag) -> {
					ContinueOperation op = new ContinueOperation();
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return null;
				});

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("break")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";"))).end((parent, bag) -> {
					System.out.println("break");
					BreakOperation op = new BreakOperation();
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return null;
				});

		StateTree<JavaElement> operation_if = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("if")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("("))).then((v) -> {
					System.out.println("inside IF");
					return true;
				}).then(new RedirectStateTree<>(value, (global, local) -> global.set(local.get())))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")"))).end((parent, bag) -> {
					System.out.println("if " + bag);
					IfOperation op = new IfOperation(bag.get());
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return op;
				});
		operation_if.then(operation);
		operation_if.then((validator) -> validator.validate((token) -> token.getValue().equals("{"))).end((a, b) -> a)
				.multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> switch_cases = new StateTree<>();
		StateTree<JavaElement> switch_case_begin = switch_cases
				.then((validator) -> validator.validate((token) -> token.getValue().equals("case")));
		StateTree<JavaElement> switch_case = switch_case_begin
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(":")));
		switch_case.then(switch_case_begin);
		switch_case.end((a, b) -> a).multiple(function_container);

		switch_cases.then((validator) -> validator.validate((token) -> token.getValue().equals("default")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(":"))).end((a, b) -> a)
				.multiple(function_container);

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("switch")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")"))
						&& validator.validate((token) -> token.getValue().equals("{")))
				.end((a, b) -> a).multiple(switch_cases)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_for = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("for")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")));

		StateTree<JavaElement> operation_for_first_part = operation_for
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";")));

		StateTree<JavaElement> operation_for_modifier = operation_for.then(new RedirectStateTree<>(modifier, null));
		operation_for_modifier.then(operation_for_modifier);

		StateTree<JavaElement> operation_for_type = operation_for
				.then(new RedirectStateTree<>(type, (global, local) -> global.set(null)));
		operation_for_modifier.then(operation_for_type);

		StateTree<JavaElement> operation_for_assign = operation_for
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME)));
		operation_for_modifier.then(operation_for_assign);
		operation_for_type.then(operation_for_assign);

		StateTree<JavaElement> operation_for_assign_end = operation_for_assign
				.then((validator) -> validator.validate((token) -> token.getValue().equals("=")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)));

		operation_for_assign.then(operation_for_assign);
		operation_for_assign.then(operation_for_first_part);

		operation_for_assign_end.then((validator) -> validator.validate((token) -> token.getValue().equals(",")))
				.then(operation_for_assign);
		operation_for_assign_end.then(operation_for_first_part);

		StateTree<JavaElement> operation_for_second_part = operation_for_first_part
				.then((validator) -> validator.validate((token) -> token.getValue().equals(";")));

		operation_for_first_part.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)))
				.then(operation_for_second_part);

		StateTree<JavaElement> operation_for_end = operation_for_second_part
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")));
		operation_for_end.<JavaElement>end((parent, bag) -> {
			ForOperation op = new ForOperation(null, null, null, null);
			if (parent instanceof OperationContainer)
				((OperationContainer) parent).addOperation(op);
			System.out.println("FOOOR");
			return op;
		}).then(operation);

		operation_for_end.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
				.<JavaElement>end((parent, bag) -> {
					ForOperation op = new ForOperation(null, null, null, null);
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return op;
				}).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		operation_for_assign.then((validator) -> validator.validate((token) -> token.getValue().equals(":")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null))).then(operation_for_end);

		StateTree<JavaElement> operation_for_update = operation_for_second_part
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)));
		operation_for_update.then(operation_for_end);
		operation_for_update.then((validator) -> validator.validate((token) -> token.getValue().equals(",")))
				.then(operation_for_update);

		StateTree<JavaElement> operation_while = operation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("while")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(null)))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")));
		operation_while.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
				.<JavaElement>end((parent, bag) -> {
					WhileOperation op = new WhileOperation(null);
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return op;
				}).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		operation_while.<JavaElement>end((parent, bag) -> {
			WhileOperation op = new WhileOperation(null);
			if (parent instanceof OperationContainer)
				((OperationContainer) parent).addOperation(op);
			return op;
		}).then(operation);

		operation.then((validator) -> validator.validate((token) -> token.getValue().equals("synchronized")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")))
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(local.get())))
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")))
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{")))
				.<JavaElement>end((parent, bag) -> {
					SynchronizedOperation op = new SynchronizedOperation(bag.get());
					if (parent instanceof OperationContainer)
						((OperationContainer) parent).addOperation(op);
					return op;
				}).multiple(function_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> operation_name = operation
				.then(new RedirectStateTree<>(value, (global, local) -> global.set(local.get())));
		operation_name.then((validator) -> validator.validate((token) -> token.getValue().equals("=")))
		.then(new RedirectStateTree<>(value, (global, local) -> global.set("newer", local.get())))
		.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
		.end((parent, bag) -> parent);
		
		operation_name.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
				.end((parent, bag) -> {
					Value action = bag.get();
					if (action instanceof BiValue) {
						BiValue assign = (BiValue) action;
						AssignOperation op = new AssignOperation(assign.left(), assign.right());
						if (parent instanceof OperationContainer)
							((OperationContainer) parent).addOperation(op);
					}
					if (action instanceof MethodCallValue) {
						MethodCallValue call = (MethodCallValue) action;
//						MethodCallOperation op = new MethodCallOperation((Value) null, call.getToken(),
//								call.getParameters());
//						if (parent instanceof OperationContainer)
//							((OperationContainer) parent).addOperation(op);
					}
					return null;
				});

		function_container.then(variable);
		function_container.then(operation);
	}

}
