package dev.peerat.parser.java.value;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.Variable.VariableContainer;
import dev.peerat.parser.java.operation.Operation;
import dev.peerat.parser.java.operation.Operation.OperationContainer;
import dev.peerat.parser.java.value.Value.ValueContainer;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class LambdaValue extends Value implements OperationContainer, VariableContainer, ValueContainer{
	
	private List<LambdaParameter> parameters;
	private List<JavaElement> operations;
	
	public LambdaValue(List<LambdaParameter> parameters){
		this.parameters = parameters;
		this.operations = new ArrayList<>();
	}
	
	public LambdaValue(List<LambdaParameter> parameters, Value value){
		this(parameters);
		this.operations.add(value);
	}
	
	public List<LambdaParameter> getParameters(){
		return this.parameters;
	}
	
	public List<JavaElement> getOperations(){
		return this.operations;
	}
	
	@Override
	public void addVariable(Variable variable){
		this.operations.add(variable);
	}

	@Override
	public void addOperation(Operation operation) {
		this.operations.add(operation);
	}
	
	@Override
	public void addValue(Value value){
		this.operations.add(value);
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder) {
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list) {
		
	}
	
	@Override
	public String toString(){
		return "[LambdaValue] "+parameters+"";
	}

	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		for(JavaElement element : this.operations){
			bag.merge(element.visit(visitor));
		}
		return bag;
	}
	
	public static class LambdaParameter{
		
		private Token type;
		private Token name;
		
		public LambdaParameter(Token type, Token name){
			this.type = type;
			this.name = name;
		}
		
		public Token getType(){
			return this.type;
		}
		
		public Token getName(){
			return this.name;
		}
		
		@Override
		public String toString(){
			return "LambdaValue[type="+type+",name="+name+"]";
		}
	}
}
