package dev.peerat.parser.java.visitor.value;

import java.util.function.Predicate;

import dev.peerat.parser.Bag;
import dev.peerat.parser.java.value.BiValue;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaBiValueVisitor extends JavaVisitor<BiValue> {
	
	private Visitor<Value> leftPredicate;
	private Predicate<String> actionPredicate;
	private Visitor<Value> rightPredicate;

	public JavaBiValueVisitor() {
		super(BiValue.class);
	}

	public JavaBiValueVisitor left(Visitor<Value> visitor) {
		leftPredicate = visitor;
		return this;
	}

	public JavaBiValueVisitor action(Predicate<String> checker) {
		actionPredicate = checker;
		return this;
	}

	public JavaBiValueVisitor right(Visitor<Value> visitor) {
		rightPredicate = visitor;
		return this;
	}

	@Override
	public VisitorBag visitElement(BiValue element){
		VisitorBag bag = new VisitorBag();
		
		if(leftPredicate != null){
			if(!leftPredicate.canVisit(element.left().getClass())) return bag;
			VisitorBag visited = leftPredicate.visit(element.left());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		if(actionPredicate != null){
			if(!actionPredicate.test(element.getAction())) return bag;
		}
		
		if(rightPredicate != null){
			if(!rightPredicate.canVisit(element.left().getClass())) return bag;
			VisitorBag visited = rightPredicate.visit(element.right());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		bag.compute(element);
		return bag;
	}
}
