package dev.peerat.parser.java.visitor.value;

import dev.peerat.parser.java.value.ModifierValue;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.java.value.Value;
import java.util.function.Predicate;

public class JavaModifierValueVisitor extends JavaVisitor<ModifierValue> {
	private Predicate<String> modifierPredicate;
	private Visitor<Value> valuePredicate;

	public JavaModifierValueVisitor() {
		super(ModifierValue.class);
	}

	public JavaModifierValueVisitor modifier(Predicate<String> checker) {
		modifierPredicate = checker;
		return this;
	}

	public JavaModifierValueVisitor value(Visitor<Value> visitor) {
		valuePredicate = visitor;
		return this;
	}

	@Override
	public VisitorBag visitElement(ModifierValue element) {
		VisitorBag bag = new VisitorBag();
		
		if(modifierPredicate != null){
			if(!modifierPredicate.test(element.getModifier().getValue())) return bag;
		}
		
		if(valuePredicate != null){
			if(!valuePredicate.canVisit(element.getValue().getClass())) return bag;
			VisitorBag visited = valuePredicate.visit(element.getValue());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		bag.compute(element);
		return bag;
	}
}
