package dev.peerat.parser.visitor;

import java.util.HashSet;
import java.util.Set;
import java.util.function.Predicate;

public class Operator{
	
	//Success ? Fail ?
	@SafeVarargs
	public static <E> Visitor<E> and(Visitor<E>... visitors){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag bag = new VisitorBag();
				Set<Object> set = new HashSet<>();
				for(Visitor<E> visitor : visitors){
					VisitorBag vbag = visitor.visit(element);
					if(!vbag.isValidated()) return new VisitorBag();
					bag.merge(vbag);
					for(Object computed : vbag.computeList()){
						set.add(computed);
					}
				}
				for(Object obj : set) bag.compute(obj);
				return bag;
			}

			@Override
			public boolean canVisit(Class<?> type){
				for(Visitor<E> visitor : visitors){
					if(visitor.canVisit(type)) return true;
				}
				return false;
			}
		};
	}
	
	//Quid de si c'est empty ?
	//Quid de si on a un collect dans les 2 ? ou dans 1 qui sourtout ne sera peux être pas appeler ?
	@SafeVarargs
	public static <E> Visitor<E> or(Visitor<E>... visitors){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag bag = new VisitorBag();
				for(Visitor<E> visitor : visitors){
					VisitorBag vbag = visitor.visit(element);
					if(!vbag.isValidated()) return new VisitorBag();
					bag.merge(vbag);
					for(Object computed : vbag.computeList()){
						bag.compute(computed);
					}
					return bag;
				}
				return bag;
			}

			@Override
			public boolean canVisit(Class<?> type) {
				for(Visitor<E> visitor : visitors){
					if(visitor.canVisit(type)) return true;
				}
				return false;
			}
		};
	}
	
	public static <E> Visitor<E> not(Visitor<E> visitor){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag result = visitor.visit(element);
				if(result.isValidated()) return null;
				VisitorBag bag = new VisitorBag();
				bag.compute(element);
				return bag;
			}
			
			@Override
			public boolean canVisit(Class<?> type){
				return visitor.canVisit(type);
			}
		};
	}
	
	public static <T extends Visitor<?>> T key(Visitor<?> visitor){
		return null;
	}
	
	public static <T extends Visitor<?>> T value(Visitor<?> visitor){
		return null;
	}
	
	public static <E> Visitor<E> collect(Visitor<E> visitor){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				return visitor.visit(element).collect();
			}

			@Override
			public boolean canVisit(Class<?> type){
//				System.out.println("collect can visit ? "+type);
				return visitor.canVisit(type);
			}
		};
	}
	
	public static <E, T extends Visitor<E>> T count(T visitor, Predicate<Integer> validator){
		return (T) new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag visited = visitor.visit(element);
				return validator.test(visited.toList().size()) ? visited : new VisitorBag();
			}

			@Override
			public boolean canVisit(Class<?> type) {
				return true;
			}
		};
	}

}
